@@ -1587,9 +1587,10 @@ fn ctx.loopck(rules) {
 		//歩けないなら始末
 		goto lb_WALL
 	::lb_WALL::
-		//ここにくるのは 1消費でjmpか0流れ着くか。
+		//ここにくるのは0jmpか未消費終端か。
+		//regexはtapeしてからcでfastmapするので怪しい判定しか出来ない
 		if(st.rc!=1){
-			st.eres=st.eres.."\ninfo: rule "..st.rpos.." "..rules[st.rpos][1].data..": all token includes skip factor (!&*? '')"
+			st.eres=st.eres.."\ninfo: rule "..st.rpos.." "..rules[st.rpos][1].data..": regex / all token includes skip factor (!&*? '')"
 			st.passrule[st.rpos]=1
 			st.rc= -1
 		}
