@@ -188,7 +188,7 @@ ped  : ped -re 'rule1 <- ![0-9] [a-zA-Z0-9_]+	{_0 = "-"}'
 @CONFORMING_TO POSIX.1-2001+
 @BUGS \-
 @COPYRIGHT Copyright 2020 momi-g, GPLv3+
-@VERSION 2021-07-05
+@VERSION 2021-07-10 v2.0.1
 @SEE_ALSO
 <pre>
 https://pdos.csail.mit.edu/papers/parsing:popl04.pdf
@@ -368,8 +368,6 @@ typedef struct peg_jumpstate_tag {
 	sg_t* instr;
 	sg_t* outstr;
 	int curpos;	//now charpos
-	int pre_curpos;	//detect loop
-	int moveck;
 //
 	int rpos;
 	int tpos;
@@ -470,13 +468,16 @@ static int calc_sendsz(ped_t* ped, int reqsz);
 //jmp
 static jbm_t ped_jmp(ped_t* ped);
 static rtoken_t* gettoken(ped_t* ped, int rpos, int tpos);
-static void res_push(ped_t* ped, int mode, int n);
+static void res_push(jst_t* jst, int mode, int n);
 static void pfix_restore(ped_t* ped);
 static int fix_result(ped_t* ped, int rc);
 static void nlpos_update(ped_t* ped);
 static int jmp_suc(ped_t* ped);
 static int jmp_fail(ped_t* ped);
 static ureg_t* getrg(ped_t* ped, int rpos, int tpos);
+static const char* jmp_seterr(ped_t* ped, const char* msg);
+static void ipos_push(jst_t* jst, int idx);
+static int ipos_pop(jst_t* jst);
 
 //edit
 static jbm_t ped_edit(ped_t* ped);
@@ -495,6 +496,8 @@ static char* my_itoa8(int num);
 
 #define RESARR_INITSZ	128		//自動伸長タイプ
 
+
+
 int fkcnt_[10]={0};
 int* clcnt = fkcnt_;
 // errはerrno!=0で戻る
@@ -512,14 +515,14 @@ ped_t* ped_new_impl(const char* mode, const char* rstr, int rstrsz, const char*
 //
 	ped_t* ped = (ped_t*)calloc(1, sizeof(ped_t) );
 	;STOP(ped==NULL, "calloc() failed");
-	
 //info
 	if( strchr(mode,'h') ){ ped->ruleinfo = ped_info("h", rstr, rstrsz); }
 	else if( strchr(mode,'H') ){ ped->ruleinfo = ped_info("H", rstr, rstrsz); }
 	else if( strchr(mode,'V') ){ ped->ruleinfo = ped_info("V", rstr, rstrsz); }
 	else if( strchr(mode,'d') ){ ped->ruleinfo = ped_info(mode, rstr, rstrsz); }
 	if( strspn(mode, "hHV") ){ return ped; }
-//maindata	
+
+//maindata
 	ped->msv = msv_new(mode, nlstr, nlstrsz);
 //	
 	errno=0;
@@ -569,10 +572,10 @@ void ped_free_impl(ped_t* ped){
 	if(ped==NULL){return;}
 //	reset_jobs(ped, -1);	//	kill fork
 // forkの前の可能性もあるのでreset系は使わずshutdown 運用でreadになってるはず
-	free((void*)ped->ruleinfo);
+	free((void*)ped->ruleinfo);		//const外し
 	free((void*)ped->emsg);
 //
-	if(ped->msv){
+	if(ped->msv && ped->msv->jobs>1){
 		int* pidarr = ped->msv->pidarr;
 		int* sfdarr = ped->msv->sfdarr;
 		for(int i=1; pidarr[i]; i++) { shutdown(sfdarr[i], SHUT_RDWR); }
@@ -589,6 +592,7 @@ void ped_free_impl(ped_t* ped){
 
 // err,別ファイルを読みたいとき。rules系とedbuf系はノータッチ
 void ped_reset(ped_t* ped) {
+	if(ped==NULL){return;}
 	reset_jobs(ped, 0);
 	msv_t* msv = ped->msv;
 //	s_emsg[0]='\0';
@@ -772,15 +776,14 @@ static msv_t* msv_new(const char* mode, const char* nlstr, int nlstrsz){
 	if(p){ jobs = (int)strtol(p, NULL, 10); }
 	if(jobs<=0){jobs=1;}
 	msv->jobs = jobs;
-	
-	msv->sfdarr = (int*)calloc(sizeof(int), jobs);	//selfの分 sentinelは止めた
+	msv->sfdarr = (int*)calloc(jobs, sizeof(int));	//selfの分 sentinelは止めた
 	; if(errno){ return NULL; }
-	msv->pidarr = (int*)calloc(sizeof(int), jobs);
+	msv->pidarr = (int*)calloc(jobs, sizeof(int));
 	; if(errno){ return NULL; }
 	msv->pidarr[0]=getpid();
 	msv->sfdarr[0]= -1;	//selfは-1固定
 
-	msv->nlstr = (char*)calloc(nlstrsz+1, 1);
+	msv->nlstr = (char*)calloc(nlstrsz+1, sizeof(char) );
 	memcpy(msv->nlstr, nlstr, nlstrsz);
 	msv->nlstrsz = nlstrsz;
 	return msv;
@@ -807,9 +810,8 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 	rdata_t* obj = (rdata_t*)calloc(1, sizeof(rdata_t) );
 	lbc_t* lvm = lbc_new();
 	obj->lvm=lvm;
-
-//opt setting
-//	obj->vflg = (strchr(mode, 'v')&&1);
+// opt setting
+// obj->vflg = (strchr(mode, 'v')&&1);
 	obj->tmode = 0;
 	if(strchr(mode, 't') ){obj->tmode=1;}
 	if(strchr(mode, 'T') ){obj->tmode=2;}	//actskipモード。生モード
@@ -861,6 +863,7 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 	obj->rhead = (void*)(uintptr_t)lbc_tbval(lvm, 0, 1, "rhead").d;
 	obj->atokens = (void*)(uintptr_t)lbc_tbval(lvm, 0, 1, "atokens").d;
 	obj->ahead = (void*)(uintptr_t)lbc_tbval(lvm, 0, 1, "ahead").d;
+
 	//rules data (after opt)
 	//set_cmntokens(obj, rtokens, atokens, rhead, ahead); // setたちに先立って生成
 //dbg(obj);
@@ -892,8 +895,8 @@ static void rdata_free_impl(rdata_t* rdata) {
 	reset_rglists(rdata);
 	reset_clists(rdata);
 	lbc_free(rdata->lvm);
-	free(rdata);
 	*rdata = (rdata_t){0};
+	free(rdata);
 }
 
 static char* set_rglists(rdata_t* rdata) {
@@ -922,57 +925,55 @@ static char* set_rglists(rdata_t* rdata) {
 	}
 //make regex
 	int mmsz = 128;
-	char* mm = (char*)malloc(mmsz);		//tmpbuff
+	char* mm = (char*)malloc(mmsz);		//tmpbuff. realloc() placeholder
 	dq_t* dq = pdq_new();
 	for(int i=0; rdata->rtokens[i].num!= -9; i++) {
 		const char* ctg = rdata->rtokens[i].ctg;
+		if(strcmp(ctg, "reg")!=0) { pdq_push(dq, NULL); continue;}
 		ureg_t* robj=NULL;
-		if(strcmp(ctg, "reg")==0) {
-			size_t rc= (size_t)-1;
-			char* sp = (char*)rdata->rtokens[i].data;
-			char* dp = mm;
-			size_t inleft = rdata->rtokens[i].num;
-			size_t outleft = mmsz-1;	//1byteでerrで対処可能にしとく
-			while(rc==(size_t)-1) {
-				errno=0;
-				iconv(fd_cv, NULL,NULL,NULL,NULL);	//reset?
-				rc = iconv(fd_cv, &sp, &inleft, &dp, &outleft);
-				if(errno==E2BIG){
-					//req mem 状態は変わってるので初期化再挑戦
-					mmsz*=2;
-					mm = (char*)realloc(mm,mmsz);
-					sp = (char*)rdata->rtokens[i].data;
-					dp = mm;
-					inleft = rdata->rtokens[i].num;
-					outleft = mmsz-1;	//1byteでerrで対処可能にしとく
-				}
-				else if(errno!=0){
-					//バイナリ混在なのでそのままコピー 最低1byteは空きがある
-					*dp= *sp;
-					inleft--;
-					outleft--;
-					sp++;
-					dp++;
-				}
+		size_t rc= (size_t)-1;
+		char* sp = (char*)rdata->rtokens[i].data;
+		char* dp = mm;
+		size_t inleft = rdata->rtokens[i].num;
+		size_t outleft = mmsz-1;	//1byteでerrで対処可能にしとく
+		while(rc==(size_t)-1) {
+			errno=0;
+			iconv(fd_cv, NULL,NULL,NULL,NULL);	//reset?
+			rc = iconv(fd_cv, &sp, &inleft, &dp, &outleft);
+			if(errno==E2BIG){
+				//req mem 状態は変わってるので初期化再挑戦
+				mmsz*=2;
+				mm = (char*)realloc(mm,mmsz);
+				sp = (char*)rdata->rtokens[i].data;
+				dp = mm;
+				inleft = rdata->rtokens[i].num;
+				outleft = mmsz-1;	//1byteでerrで対処可能にしとく
 			}
-			robj = ureg_new_raw(mm, dp-mm);	//m ,sz
-			if(robj==NULL){
-				int rpos=0;
-				for(;;rpos++){if(i< rdata->rhead[rpos]-1){break;} }
-				rpos--;
-				int ridx = rdata->rhead[rpos]-1;	//-1 lua分
-				int tpos = i - ridx;	//一個前 tposは1以上なので+1は不要
-				snprintf(ebuf, 127, "regex() failed: rule:%d fld:%d info:%s\n"
-					, rpos+1, tpos, rdata->rtokens[ridx+tpos].data);
-				emsg=ebuf;	
-				break;
+			else if(errno!=0){
+				//バイナリ混在なのでそのままコピー 最低1byteは空きがある
+				*dp= *sp;
+				inleft--;
+				outleft--;
+				sp++;
+				dp++;
 			}
 		}
+		robj = ureg_new_raw(mm, dp-mm);	//m ,sz
+		if(robj==NULL){
+			int rpos=0;
+			for(;;rpos++){if(i< rdata->rhead[rpos]-1){break;} }
+			rpos--;
+			int ridx = rdata->rhead[rpos]-1;	//-1 lua分
+			int tpos = i - ridx;	//一個前 tposは1以上なので+1は不要
+			snprintf(ebuf, 127, "regex() failed: rule:%d fld:%d info:%s\n"
+				, rpos+1, tpos, rdata->rtokens[ridx+tpos].data);
+			emsg=ebuf;	
+			break;
+		}
 		pdq_push(dq, robj);
 	}
-
-	pdq_push(dq, NULL);
-	rdata->rglists = (ureg_t**)pdq_2arr(dq);
+	pdq_push(dq, NULL);	//-9のはずだが一応ぬるぽを突っこんどく
+	rdata->rglists = (ureg_t**)pdq_2arr(dq);	//ポインタ番号の羅列
 	iconv_close(fd_cv);
 	free(mm);
 	pdq_free(dq);
@@ -1027,11 +1028,12 @@ static const char* set_clists(rdata_t* rdata) {
 	int erpos = 0;	//内部ruleも含めた全ルールのsentinel
 	for(; rdata->rhead[erpos]!= -9; erpos++);;
 	int* allmk = calloc(sizeof(int), erpos);	//esepはclist無し
-	
+
+	//pass1 下調べ
 	dq_t* obj = pdq_new();
 	for(int rpos=0; rpos<erpos; rpos++) {
 		//bit演算を使えばflg系が少し楽
-		char* arr = calloc(256, 1);
+		char* arr = (char*)calloc(256, sizeof(char));
 		int idx = rdata->rhead[rpos] -1;	//lua分 
 		int tpos = 1; 	//1スタート 0はheadが入る
 		for(;;tpos++){
@@ -1054,9 +1056,10 @@ static const char* set_clists(rdata_t* rdata) {
 			if(tk.ctg[0]=='c'||tk.ctg[0]=='a') { CBIT_SET(arr, tk.data, 256); }
 			if(tk.suf[0]=='*'||tk.suf[0]=='?'||(tk.ctg[0]=='l'&&tk.num==0)) {continue;}
 			//NEXT可能なのは*?復活系と""!だけ。&はそこで終了
-			//全部*?系で未消費でオワタ >>sepに突っこんで自動break
-			if(tk.ctg[0]=='s') {allmk[rpos]=1;}
-			//sepとかesep,eofもあるけどcharカウントには無関係
+			
+			//全部*?系で未消費でオワタ >>sepに突入するので自動break
+			if(tk.ctg[0]=='s') {allmk[rpos]=1;}	//未消費可能性を立てておく
+			//残骸はsepとかesep,eofもあるけどcharカウントには無関係
 			break;
 		}
 		pdq_push(obj, arr);
@@ -1121,11 +1124,11 @@ static void reset_clists(rdata_t* rdata) {
 static jst_t* jst_new(const char* nlstr, int nlstrsz){
 	char* emsg=NULL;
 //
-	jst_t* res = (jst_t*)calloc(1, sizeof(jst_t) );
+	jst_t* res = (jst_t*)calloc(1, sizeof(jst_t) );		//ここでmemerrが起きる
 	; if(errno){ emsg=strerror(errno); goto lb_ERR; }
-	res->resarr = (resarr_t*)calloc(sizeof(resarr_t), RESARR_INITSZ);
+	res->resarr = (resarr_t*)calloc( RESARR_INITSZ, sizeof(resarr_t) );
 	; if(errno){ emsg=strerror(errno); goto lb_ERR; }
-	res->ipos_mem = (int*)calloc(sizeof(int), RESARR_INITSZ);
+	res->ipos_mem = (int*)calloc( RESARR_INITSZ, sizeof(int) );
 	; if(errno){ emsg=strerror(errno); goto lb_ERR; }
 
 	res->instr = sg_new();
@@ -1162,8 +1165,14 @@ static void jst_reset(jst_t* jst){
 	jst->tpos=1;
 	jst->seof=0;
 	jst->resepos = -1;
-	jst->ipos_c = 0;
-	jst->ipos_mem[0]=-1;
+	jst->ipos_c = -1;
+	jst->ipos_mem[0]= 0;
+
+	//ここでres_push()したいけどpedがないからうんこちゃん
+	//初回。情報追加しておく。curposとかidentとか重要マーカー情報があちこちで
+	//必要なのでvecセット
+	res_push(jst, TK_MARK, -9);
+	ipos_push(jst, jst->resepos);	//start位置	//eposもiposも初期-1なのでsetで0initになる
 }
 
 static edbuf_t* edbuf_new(void){
@@ -1182,6 +1191,7 @@ static void edbuf_free(edbuf_t* obj){
 	sg_free(obj->sg1);
 	sg_free(obj->sg2);
 	sg_free(obj->sgbuf);
+	free(obj);
 }
 
 
@@ -1201,9 +1211,8 @@ static rtoken_t* gettoken(ped_t* ped, int rpos, int tpos) {
 }
 
 //高速化のためdqより直接利用の方が効果的
-//呼び出しが1000万回クラスになるのでクリティカル扱い
-static void res_push(ped_t* ped, int mode, int n) {
-	jst_t* jst = ped->jst;
+//呼び出しが1000万回クラスで10倍圧倒的topなのでクリティカル扱い
+static void res_push(jst_t* jst, int mode, int n) {
 	jst->resepos++;
 	if(jst->resmaxsz <= jst->resepos) {
 		jst->resmaxsz=jst->resmaxsz*2;
@@ -1217,22 +1226,26 @@ static void res_push(ped_t* ped, int mode, int n) {
 	//構造体初期化子で作成しないと代入できない。さらに構造体の演算子は=コピーだけ
 }
 
-//resarrの区切りはLIT/RTN/LOOPENDの三つanyもeofもLIT. RTNとLOOPENDは遠くまで戻る
-//直接くるのはLITとLOOPEND、suc,failの途中割り込みでRTNもくる
+// !&巻き戻し
+//resarrの区切りはLIT/RTN/LOOPENDの三つanyもeofもregもLIT. RTNとLOOPENDは遠くまで戻る
+//LITとLOOPENDがダイレクトで、suc,failの途中割り込みでRTNもくる
 static void pfix_restore(ped_t* ped) {
 	//restoreにくるのはsucのみ
-	jst_t* st = ped->jst;
-	resarr_t tk = st->resarr[st->resepos];
+	jst_t* jst = ped->jst;
+	resarr_t tk = jst->resarr[jst->resepos];
 	if(tk.mode == TK_LIT) {
 		//lit系。tokenは一つだけの単純戻し。
-		st->curpos = tk.curpos;
-		st->resepos--;
+		jst->curpos = tk.curpos;
+		jst->resepos--;
 		return;
 	}
 	if(tk.mode == TK_RTN) {
-		st->resepos = st->ipos_mem[st->ipos_c+1];	//成功rtn後なのでident位置のはず
-		st->curpos = st->resarr[st->resepos].curpos;
-		st->resepos--;
+		jst->resepos = jst->ipos_mem[jst->ipos_c+1];
+		//成功rtn直後で+1のmemにはjmp直前の状態が入ってるので利用する
+		//スタック的には保証不可だけど触らないようにしてるのでいけるだろう
+		//pushする前だから残ってるしそのためのメモだし
+		jst->curpos = jst->resarr[jst->resepos].curpos;
+		jst->resepos--;
 		return;
 	}
 	//loop系の締切りは面倒. loopとrtnの混合になる
@@ -1240,30 +1253,30 @@ static void pfix_restore(ped_t* ped) {
 	//重いがsuc自体が少なくloop系かつ!&なのでそれほど負担にはならないはず
 	if(tk.mode == TK_LOOPEND) {
 		//a,b,lp,x,lp,x,lp,x,endで切れるまで辿る. aの完了まで
-		st->resepos--;	//LOOP
+		jst->resepos--;	//LOOP
 		while(1) {
-			st->resepos--;	//LIT or RTN
-			int ck = st->resarr[st->resepos].mode;
+			jst->resepos--;	//LIT or RTN
+			int ck = jst->resarr[jst->resepos].mode;
 			if(ck==TK_RTN) {
 				int cnt=1;
 				while(cnt) {
-					st->resepos--;
-					ERRact(st->resepos<0, "fatal err",exit(1));
-					int cck = st->resarr[st->resepos].mode;
+					jst->resepos--;
+					ERRact(jst->resepos<0, "fatal err",exit(1));
+					int cck = jst->resarr[jst->resepos].mode;
 					if(cck==TK_RTN) {cnt++;}
 					if(cck==TK_IDENT) {cnt--;}
 					//IDENTは残る
 				}
 			}
 			//LIT+共通
-			st->resepos--;	//判定用に一回下げる
-			if(st->resepos<0) {break;}	//topならおしまい
-			if(st->resarr[st->resepos].mode == TK_LOOP) {continue;}
+			jst->resepos--;	//判定用に一回下げる
+			if(jst->resepos==0) {break;}	//topならおしまい
+			if(jst->resarr[jst->resepos].mode == TK_LOOP) {continue;}
 			break;
 		}
-		st->resepos++;		//curを取得して巻き戻し
-		st->curpos = st->resarr[st->resepos].curpos;
-		st->resepos--;
+		jst->resepos++;		//curを取得して巻き戻し
+		jst->curpos = jst->resarr[jst->resepos].curpos;
+		jst->resepos--;
 		return;
 	}
 }
@@ -1272,37 +1285,46 @@ static int fix_result(ped_t* ped, int rc) {
 	//大変。suf/preの分類.tokenの体裁も整える rtn 0/1/2 loop, suc,fail,eof 色々
 	rtoken_t* tk = gettoken(ped, -1, -1);
 	jst_t* jst = ped->jst;
-	// suc,fail,loop,rollbk..loop系の打ち切りはfailで+*が露出してる
-//suc+loop. !ID* なら*が優先
-//	if(rc== -1 && strpbrk(tk->suf,"*+") ) {
-// 自前. 20%近く改善した
-	if(rc== -1 && (tk->suf[0]=='*' || tk->suf[0]=='+')) {
-		res_push(ped, TK_LOOP, 0);
+	if(rc== -1 && (tk->suf[0]=='*'||tk->suf[0]=='+') ) {
+		// 成功でも未消費がありえる 未消費loopは散らばってて
+		// 厄介。accとここ。
+		// ここではident以外の単体系のみkickする
+		//単体系で成功ってことは文字が入ってないといけない
+		//loop系で eof*もありえるけど明らかに不正なのでkickでいい
+		//単体系はreseposに直前が露出してるので直接観測できる
+		if(tk->ctg[0]!='i' && jst->resarr[jst->resepos].curpos == jst->curpos){
+			jmp_seterr(ped, "detect 0 byte consume term");
+			return -20;
+		}
+		res_push(ped->jst, TK_LOOP, 0);
 		return 0;
 	}
 //救済。loop系はval/lp,val,lp,val/ 系で?も失敗はval""が欲しいので""追加
 	if(rc== -2 && tk->suf[0]=='?') {
-		res_push(ped, TK_LIT, 0);
+		res_push(ped->jst, TK_LIT, 0);
 		rc= -1;
 	}
 	else if (rc== -2 && tk->suf[0]=='*') {
 		//resepos== -1もある 初期値で存在しない場合。完全勝利のとき
-		int modeck = !TK_LOOP;
-		if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
+		//>>初期resepos==0をMARKで追加して不要になった
+	//	int modeck = !TK_LOOP;
+	//	if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
+		int modeck =jst->resarr[jst->resepos].mode;
 		if(modeck!=TK_LOOP) {
 			//*だが失敗即終了...lit/loop/loope下駄 loopなら二つ前がloopのはず
-			res_push(ped, TK_LIT, 0);
-			res_push(ped, TK_LOOP, 0);
+			res_push(ped->jst, TK_LIT, 0);
+			res_push(ped->jst, TK_LOOP, 0);
 		}
-		res_push(ped, TK_LOOPEND, 0);	//LITでもいいけど!&の巻き戻しで分かりやすくなる
+		res_push(ped->jst, TK_LOOPEND, 0);	//LITでもいいけど!&の巻き戻しで分かりやすくなる
 		rc = -1;
 	}
 	else if (rc== -2 && tk->suf[0]=='+') {
 // '+' だけ失敗がある
-		int modeck = !TK_LOOP;	//reseposは-1でidxエラーがある
-		if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
+//		int modeck = !TK_LOOP;	//reseposは-1でidxエラーがある
+//		if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
+		int modeck = jst->resarr[jst->resepos].mode;
 		if(modeck!=TK_LOOP) { rc= -2;}
-		else { res_push(ped, TK_LOOPEND, 0); rc= -1; }
+		else { res_push(ped->jst, TK_LOOPEND, 0); rc= -1; }
 	}
 
 // !&ならば共通巻き戻し
@@ -1314,7 +1336,7 @@ static int fix_result(ped_t* ped, int rc) {
 		if(tk->pre[0]=='!' && rc==-1) { rc= -2; }	//成功を踏み潰す 
 		else if(tk->pre[0]=='!' && rc== -2) { rc= -1;  }
 		//ダミーが必要なら突っこむ restoreで全削除してるため
-		if(rc== -1){ res_push(ped, TK_LIT, 0); }
+		if(rc== -1){ res_push(ped->jst, TK_LIT, 0); }
 	}
 	return rc;
 }
@@ -1347,8 +1369,8 @@ static void nlpos_update(ped_t* ped) {
 
 //idx==0はinitかMARKで初回位置を保持する 巻き戻しで必要
 //jmpしていればipos_cが1以上になる 0ならjmp無しでvicとかrootとか
-static void ipos_push(ped_t* ped, int idx) {
-	jst_t* jst = ped->jst;
+//初期代入は
+static void ipos_push(jst_t* jst, int idx) {
 	jst->ipos_c++;
 	jst->ipos_mem[jst->ipos_c] = idx;
 	if(jst->ipos_e <= jst->ipos_c) {
@@ -1358,8 +1380,7 @@ static void ipos_push(ped_t* ped, int idx) {
 	}
 	//構造体初期化子で作成しないと代入できない。さらに構造体の演算子は=コピーだけ
 }
-static int ipos_pop(ped_t* ped) {
-	jst_t* jst = ped->jst;
+static int ipos_pop(jst_t* jst) {
 	jst->ipos_c--;
 	; STOP(jst->ipos_c<-1, "fatal err");
 	return jst->ipos_mem[jst->ipos_c+1];
@@ -1368,7 +1389,6 @@ static int ipos_pop(ped_t* ped) {
 static const char* jmp_seterr(ped_t* ped, const char* msg){
 	int svrpos = ped->jst->rpos;
 	jst_t* jst = ped->jst;
-	
 	const char* bin = sg_ptr(jst->instr)+jst->curpos;
 	
 	int presz = jst->curpos;
@@ -1379,9 +1399,9 @@ static const char* jmp_seterr(ped_t* ped, const char* msg){
 	nlpos_update(ped);
 	sg_clear(ped->edbuf->sgbuf);
 	sg_addf(ped->edbuf->sgbuf
-	, "jmpERR: rule %d %s: input: %d byte/line %d/pos %d: %s: %.*s@%.*s"
-	, svrpos+1, gettoken(ped, svrpos, 0)->data, ped->msv->bcnt + jst->bcnt
-	, ped->msv->nlcnt + jst->nlcnt, jst->ccnt, msg
+	, "jmpERR: rule %d %s: l %d,%d(%d byte): %s: %.*s@%.*s"
+	, svrpos+1, gettoken(ped, svrpos, 0)->data, ped->msv->nlcnt + jst->nlcnt
+	, jst->ccnt, ped->msv->bcnt + jst->bcnt, msg
 	, presz, bin-presz, pstsz, bin
 	);
 	ped->jst->emsg = sg_ptr(ped->edbuf->sgbuf);
@@ -1393,37 +1413,47 @@ static int jmp_suc(ped_t* ped) {
 	jst_t* jst = ped->jst;
 	int rc=0;	//基本は進むだけ rcは0,1,10, 11		0はfall-through
 	jst->tpos++;	//endはact位置に相当させるのが順当
+
 	//acc / vic判定	drule作成時にendに-1のtokenを付加してる
 	if( gettoken(ped, -1, -1)->num<0 ) {
-		int svrpos = jst->rpos;	//err用
-		if(jst->ipos_c>0) {
-		// acc マーカー追加と rpos/tpos戻り
-			res_push(ped, TK_RTN, 0);
+// 戻り先を覗き見. curが進んでないのに+*系だったら永久になるので弾く
+// 2団jmpでも最終的に*+系にたどり着くのでそこでkickできる
+// jmp系永久はここで弾ける reseposをpresetならpre_curposも始末出来るか
+		//jmp直後にipos_push(,...resepos)してるので.curposが取り出せる
+		//epos情報はaccもvicも共通で使うので先取り出来る？
+		int pos = ipos_pop(ped->jst);
+		resarr_t res = jst->resarr[pos];
+		if(res.curpos == jst->curpos){
+			rtoken_t* rtk = gettoken(ped, res.rpos, res.tpos);
+			if(rtk->suf[0]=='*'||rtk->suf[0]=='+'){
+				rc= -20;
+				jmp_seterr(ped, "detect 0 byte consume");
+				goto lb_RTN;
+			}
+		}
+		if(jst->ipos_c>=0) {
+			// acc マーカー追加と rpos/tpos戻り
+			res_push(ped->jst, TK_RTN, 0);
 			//rpos巻き戻し accは成功して戻りで再処理が必要なのでrc=0ではなく-1
-			int pos = ipos_pop(ped);
-			jst->rpos = jst->resarr[pos].rpos;
-			jst->tpos = jst->resarr[pos].tpos;
+			jst->rpos = res.rpos;
+			jst->tpos = res.tpos;
 			rc= -1;
-			goto lb_RTN;
 		} else {
 			//vic
-			res_push(ped, TK_MARK, -9);
-			ipos_pop(ped);	//ipos -1
-			ipos_push(ped, jst->resepos);	//start位置入れ直し
+			res_push(ped->jst, TK_MARK, -9);
+			ipos_push(ped->jst, jst->resepos);
+			//iposが-1になるのでcurpos目当てにstart位置入れ直し
+			//rposはeditでcurrentのruleを探すのに使うのでrpos=0前に保存
+			//iposはcurのみチェックするのでrposがズレてても問題ない
+			//一致させておきたいところだけど
 			rc=0;
 			jst->rpos=0;
 			jst->tpos=1;
 			//量によって継続かeditか異なる
 			if(jst->curpos > ped->msv->bufsz_min ){ rc= -11; }	//報告なら-11, 継続は0
 			else if(jst->curpos<0) { rc=-10; jst->seof=2; }	//EOFヒット !. {_0 = "gw"} とか
-			//致命傷 *が無限ループしてる
-			if(jst->curpos==jst->pre_curpos && jst->seof!=2) {
-				rc= -20;
-				jst->rpos = svrpos;
-				jmp_seterr(ped, "detect 0 byte consume");
-			}
-			jst->pre_curpos = jst->curpos;
 		}
+		goto lb_RTN;
 	}
 lb_RTN:;
 	return rc;
@@ -1453,7 +1483,7 @@ static int jmp_fail(ped_t* ped) {
 			goto lb_RTN;
 		}
 		// next無し失敗 ... ident失敗として差し戻しのfail再判定
-		ipos_pop(ped);	//失敗確定
+		ipos_pop(ped->jst);	//失敗確定
 		jst->rpos = jst->resarr[jst->resepos].rpos;
 		jst->tpos = jst->resarr[jst->resepos].tpos;
 		jst->resepos--;	//rposがidentなのでresultから削除
@@ -1462,38 +1492,36 @@ static int jmp_fail(ped_t* ped) {
 	}
 
 	// root nextrule
-	if(1) {
-		jst->rpos++;
-		jst->tpos=0;
-		// %Rを越えたかruleを全部使い果たした
-		// 事前に%RULEでlastpos設定するけど、pegとしては全部のend判定が必要
-		// dtokenで渡すときにendが->numはidentだけど<0になるようにしてる
-		//通常identはjmp先のrule番号@luaだけど
-		if( ped->rdata->last_rpos < jst->rpos || gettoken(ped, -1,-1)->num<0 ) {
-			// EOF消費済みの正当勝利。failに入る前に切りたいけどEOF+があるので判別不能
-			if(jst->curpos<0) {jst->seof=2; rc= -10; goto lb_RTN;}
-			//EOF以外。指示がない場合で全文字消費なら完全成功
-			//ぴったりでeofが立ってない場合は確認。endruleなので一歩手前に戻す
-			//大抵はlitがいるのでここに落ちる前にseofが立ちそうだけど
-			int left = sg_sz(jst->instr) - jst->curpos;
-			if(jst->seof==0 && left==0) {
-				jst->rpos--;
-				jst->tpos=1;
-				rc=ped->msv->bufsz_min;
-				goto lb_RTN;
-			}
-			//消費済み勝利 req -1と地道収集でcurpos<0にならない場合が有る 初回とか
-			if(jst->seof>=1 && left==0) {jst->seof=2; rc= -10; goto lb_RTN;}
-			//それ以外は完全失敗。残りがいるのにruleを使い切った
-			rc= -20;
-			jmp_seterr(ped, "detect no pegrule byte input");
+	jst->rpos++;
+	jst->tpos=0;
+	// %Rを越えたかruleを全部使い果たした
+	// 事前に%RULEでlastpos設定するけど、pegとしては全部のend判定が必要
+	// dtokenで渡すときにendが->numはidentだけど<0になるようにしてる
+	//通常identはjmp先のrule番号@luaだけど
+	if( ped->rdata->last_rpos < jst->rpos || gettoken(ped, -1,-1)->num<0 ) {
+		// EOF消費済みの正当勝利。failに入る前に切りたいけどEOF+があるので判別不能
+		if(jst->curpos<0) {jst->seof=2; rc= -10; goto lb_RTN;}
+		//EOF以外。指示がない場合で全文字消費なら完全成功
+		//ぴったりでeofが立ってない場合は確認。endruleなので一歩手前に戻す
+		//大抵はlitがいるのでここに落ちる前にseofが立ちそうだけど
+		int left = sg_sz(jst->instr) - jst->curpos;
+		if(jst->seof==0 && left==0) {
+			jst->rpos--;
+			jst->tpos=1;
+			rc=ped->msv->bufsz_min;
 			goto lb_RTN;
 		}
-		//endじゃないなら次に挑戦
-		jst->tpos=1;
-		rc=0;
+		//消費済み勝利 req -1と地道収集でcurpos<0にならない場合が有る 初回とか
+		if(jst->seof>=1 && left==0) {jst->seof=2; rc= -10; goto lb_RTN;}
+		//それ以外は完全失敗。残りがいるのにruleを使い切った
+		rc= -20;
+		jmp_seterr(ped, "detect no pegrule byte input");
 		goto lb_RTN;
 	}
+	//endじゃないなら次に挑戦
+	jst->tpos=1;
+	rc=0;
+	goto lb_RTN;
 lb_RTN:;
 	ERRact(rc== 1, "fatal", exit(1));
 	return rc;
@@ -1504,9 +1532,12 @@ static jbm_t ped_jmp(ped_t* ped) {
 	// cur初期化とかはworkerの依頼時にこのfuncの外部で処理する
 	jbm_t res={0};
 	jst_t* jst = ped->jst;
+	
+	
 	//mainloop
 	int rc=0;
 	while(1) {
+//fprintf(stderr, "%d %d %d\n", jst->rpos, jst->tpos, jst->curpos);
 		// suc/fail -10/-20, req -1:all >0 sz 
 		if(rc>0||rc== -30) {break;}	//文字要求系
 		// fix処理
@@ -1526,9 +1557,6 @@ static jbm_t ped_jmp(ped_t* ped) {
 		if(jst->seof==0 && leftsz==0) { rc= BLKsz; continue; } //適当に追加
 		rtoken_t* tk = gettoken(ped, -1,-1);
 		//purne
-if(jst->rpos==18){
-//	dbg(jst->curpos, jst->instr);
-}
 		if(jst->tpos== 1 && leftsz>0 ) {
 			int c = (uint8_t)sg_ptr(jst->instr)[jst->curpos];
 			int rc = ped->rdata->clists[jst->rpos][c];	// cfilter
@@ -1542,10 +1570,10 @@ if(jst->rpos==18){
 		if(tk->ctg[0]=='i') {
 			//ident
 			int rpos_dst = tk->num -1;	//luaは1startなのでずれる
-			res_push(ped, TK_IDENT, rpos_dst);	//jmp先numを突っ込むが実際は使わない
+			res_push(ped->jst, TK_IDENT, rpos_dst);	//jmp先numを突っ込むが実際は使わない
 			jst->rpos=rpos_dst;
 			jst->tpos=1;
-			ipos_push(ped, jst->resepos);	//jmpメモ
+			ipos_push(ped->jst, jst->resepos);	//jmpメモ
 			continue; //pre系はaccで戻ってきてからs.resarrで処理する rc=0でいい
 		}
 		if(tk->ctg[0]=='l') {
@@ -1559,7 +1587,7 @@ if(jst->rpos==18){
 //dbg( sg_ptr(jst->instr)+jst->curpos, sg_ptr(jst->instr), tk->data, litsz);
 			rc = strncmp( sg_ptr(jst->instr)+jst->curpos, tk->data, litsz);
 			if(rc!=0) { goto lb_FAIL; }
-			res_push(ped, TK_LIT, litsz);
+			res_push(ped->jst, TK_LIT, litsz);
 			jst->curpos += litsz;
 			goto lb_SUC;
 		}
@@ -1571,7 +1599,7 @@ if(jst->rpos==18){
 				rc= 1;	//一文字追加要求
 				continue;
 			}
-			res_push(ped, TK_LIT, litsz);	//anyは1文字のlitとして扱う
+			res_push(ped->jst, TK_LIT, litsz);	//anyは1文字のlitとして扱う
 			jst->curpos+=litsz;
 			goto lb_SUC;
 		}
@@ -1587,7 +1615,7 @@ if(jst->rpos==18){
 			int cnum = (uint8_t)p[0];
 			rc = tk->data[cnum];
 			if(rc==0) {goto lb_FAIL;}
-			res_push(ped, TK_LIT, litsz);
+			res_push(ped->jst, TK_LIT, litsz);
 			jst->curpos += litsz;
 			goto lb_SUC;
 		}
@@ -1599,7 +1627,7 @@ if(jst->rpos==18){
 			if(rc<0){ goto lb_FAIL; }
 //dbg(rg->p, rg->sz);sleep(1);
 			int sz = rg->sz;
-			res_push(ped, TK_LIT, sz);
+			res_push(ped->jst, TK_LIT, sz);
 			jst->curpos += sz;
 			goto lb_SUC;
 		}
@@ -1611,7 +1639,7 @@ if(jst->rpos==18){
 			if(leftsz==0&&jst->seof==0) { rc = BLKsz; continue; }
 			if(leftsz!=0) { goto lb_FAIL; }
 			//eofヒット
-			res_push(ped, TK_LIT, 0);	//actもあるのでダミーを突っ込む
+			res_push(ped->jst, TK_LIT, 0);	//actもあるのでダミーを突っ込む
 			jst->curpos = -jst->curpos -1;
 			//EOF curはresに保存済でrollbackで自動的復帰
 			//バッファとして使うけど0が-+0なので-1で一つずらす
@@ -1679,7 +1707,8 @@ return rtn;
 	int lnum=0;
 	int topflg=1;
 	//resarrから編集型に整理する 速く分散したいのでjmpでは最低限の操作のみ
-	for(int i=0;i<=jst->resepos;i++) {
+	//先頭にはMARKが入ってるのでスキップ
+	for(int i=1;i<=jst->resepos;i++) {
 		resarr_t res = jst->resarr[i];
 		if(topflg){
 			edq_push(edq,(ed_t){ INFO_JO,res.rpos+1,res.tpos,SRC_PTR,0} );
@@ -1794,7 +1823,6 @@ static char* edit_rstop(ped_t* ped, int rnum, char* bin, int binsz, int cur){
 	{
 		jst_t* jst = ped->jst;
 		char* sp = bin;
-		char* spp = sp;
 		char nlc = jst->nlstr[0];	//高速化のため1byteで先行テスト
 		int nlsz = jst->nlstrsz;
 		while(sp<bin+cur) {
@@ -2666,6 +2694,7 @@ int main(){
 2021-07-10  Momi-g	<dmy@dmy.dmy>
 
 	* ped.c (set_clists): fix grep ident fastmap, limit var idx >> nidx.
+	* (jst_jmp): fix 0 consume check logic.
 
 2021-07-05  Momi-g	<dmy@dmy.dmy>
 
